package com.massa.mrules.droolsfight.tests.mrules;

import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.mrules.CustomProperties;
import com.massa.mrules.builder.MRulesBuilder;
import com.massa.mrules.context.compile.CompilationLevel;
import com.massa.mrules.context.execute.MExecutionContext;
import com.massa.mrules.droolsfight.beans.MeasureBean;
import com.massa.mrules.droolsfight.tests.AbstractTest;
import com.massa.mrules.droolsfight.tests.PrepareBeans;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.extensions.dsl.builder.MrlCustomProperties;
import com.massa.mrules.factory.ser.SerializationFactory;
import com.massa.mrules.factory.xml.RichXMLFactory;
import com.massa.mrules.set.IMruleExecutionSet;
import com.massa.mrules.set.MRuleExecutionSet;
import com.massa.util.LicenceInfoHelper;

import org.junit.Assert;
import org.junit.Assume;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RunWith(Parameterized.class)
public class MRulesTest extends AbstractTest {

	private static final Log LOG = LogFactory.getLog(MRulesTest.class);

	final String file;

	public MRulesTest(final String file) {
		Assume.assumeFalse("DSL is not licenced.", file.endsWith("mrl") && !LicenceInfoHelper.isModuleLicenced("DSL"));
		this.file = file;
	}

	@Parameters(name="{0}")
	public static Collection<Object[]> configs() {
		return Arrays.asList(new Object[][] {
			{"mrules_dt.xml"},
			{"mrules_dt_withaddon.xml"},
			{"mrules_dt.mrl"},
			{"mrules_dt_withaddon.mrl"}
		});
	}

	@Test
	public void testLight() throws Exception {
		this.test(CompilationLevel.LIGHT, false);
	}

	@Test
	public void testStd() throws Exception {
		this.test(CompilationLevel.STANDARD, false);
	}

	@Test
	public void testLightNoCache() throws Exception {
		this.test(CompilationLevel.LIGHT, true);
	}

	@Test
	public void testStdNoCache() throws Exception {
		this.test(CompilationLevel.STANDARD, true);
	}

	public void test(final CompilationLevel level, final boolean nocache) throws Exception {

		final List<MeasureBean> beans = PrepareBeans.getAllBeans();


		MRulesTest.LOG.info("RUNNING MRULES BEGIN (" + this.file + ", " + level + ", " + (nocache ? "CACHE DISABLED" : "CACHE ENABLED") + ")");

		final long t0 = System.currentTimeMillis();
		final IMruleExecutionSet set = this.buildSet(level, nocache);

		final MExecutionContext<MeasureBean> ctxt = new MExecutionContext<>(set);
		final long t1 = System.currentTimeMillis();

		int idx = 0;
		for (final MeasureBean bean: beans) {
			ctxt.setInput(bean);
			ctxt.execute();

			this.checkResult(MRulesTest.LOG, bean, idx);
			++idx;
		}

		MRulesTest.LOG.info("RUNNING MRULES END");

		MRulesTest.LOG.info("warmup ...");
		for (int i = 0 ; i < 10000 ; i++ ) {
			for (final MeasureBean bean: beans) {
				bean.reset();
				ctxt.setInput(bean);
				ctxt.execute();
			}
		}

		MRulesTest.LOG.info("RUNNING PERF MRULES BEGIN (" + this.file + ", " + level + ", " + (nocache ? "CACHE DISABLED" : "CACHE ENABLED") + ")");
		final long t2 = System.currentTimeMillis();
		for (int i = 0 ; i < 10000 ; i++ ) {
			for (final MeasureBean bean: beans) {
				bean.reset();
				ctxt.setInput(bean);
				ctxt.execute();
			}
		}
		MRulesTest.LOG.info("Time for compilation : " + (t1 - t0));
		MRulesTest.LOG.info("Time for 10000 tests : " + (System.currentTimeMillis() - t2));
		MRulesTest.LOG.info("RUNNING PERF MRULES END");
		MRulesTest.LOG.info("");

		//Read / Write tests.
		MRulesTest.LOG.info("Human readable version:\n" + set.toString());

		final StringWriter w = new StringWriter();
		final RichXMLFactory factory = new RichXMLFactory(level);
		factory.write(w, set);
		MRulesTest.LOG.info(w.toString());

		final ByteArrayOutputStream o = new ByteArrayOutputStream();
		final SerializationFactory sfactory = new SerializationFactory(level);
		sfactory.write(o, set);

		Assert.assertEquals(set, set.clone());
		final IMruleExecutionSet readFromXml = factory.read(new StringReader(w.toString()));
		Assert.assertEquals(set.toString(), readFromXml.toString());
		Assert.assertTrue(((MRuleExecutionSet) set).equals(readFromXml));
		Assert.assertEquals(set, sfactory.read(new ByteArrayInputStream(o.toByteArray())));
	}

	public IMruleExecutionSet buildSet(final CompilationLevel level, final boolean nocache) throws MConfigurationException {
		MRulesBuilder.removeEngines("MEASURE_VALIDATION");

		final Map<String, String> parameters = new HashMap<>();
		parameters.put(CustomProperties.ruleengine_uri.getLabel(), "MEASURE_VALIDATION");
		parameters.put(CustomProperties.configholder_compilation_level.getLabel(), level.name());
		parameters.put(CustomProperties.ruleenginebuilder_checkhash.getLabel(), "true");

		if (this.file.endsWith("xml")) {
			//XML factory
			parameters.put(CustomProperties.configholder_impl.getLabel(), "com.massa.mrules.builder.XmlRuleEngineConfigHolder");
			parameters.put(CustomProperties.configholder_factoryimpl.getLabel(), "com.massa.mrules.factory.xml.RichXMLFactory");
			parameters.put(CustomProperties.xmlconfigholder_xmlfile.getLabel(), this.file);
		} else {
			// MRL factory
			parameters.put(CustomProperties.configholder_impl.getLabel(), "com.massa.mrules.extensions.dsl.builder.MrlRuleEngineConfigHolder");
			parameters.put(CustomProperties.configholder_factoryimpl.getLabel(), "com.massa.mrules.extensions.dsl.factory.NativeLanguageFactory");
			parameters.put(MrlCustomProperties.mrlconfigholder_mrlfile.getLabel(), this.file);
		}

		if (nocache) {
			parameters.put(CustomProperties.configholder_cache_enabled.getLabel(), "false");
		}

		final MRulesBuilder builder = new MRulesBuilder(parameters);
		return builder.getRuleExecutionSetInstance();
	}
}
